arch snes.cpu

//-------------------------------------------------------------------------------
// SRAM expansion 1.1
// by madsiur
// release date: 10/04/2018
// apply with bass
//-------------------------------------------------------------------------------
// slot 1 expanded to:  $306A00-$307FFF
//                      $306000-$3069FF = $1600-$1FFF
//
// slot 2 expanded to:  $316A00-$317FFF
//                      $316000-$3169FF = $1600-$1FFF
//
// slot 3 expanded to:  $326A00-$327FFF
//                      $326000-$3269FF = $1600-$1FFF
//
// other SRAM values:   $336000-$3375FF (game common SRAM)
//                      $337600-$337EFF (unused)
//                      $337F00 = last altered savefile
//                      $337F01 = rng index
//                      $337F08 = checksum value #$E41B (slot 1)
//                      $337F0A = checksum value #$E41B (slot 2)
//                      $337F0C = checksum value #$E41B (slot 3)
//                      $337F0E = checksum value #$E41B (slot 4) (useless but ok...)
//-------------------------------------------------------------------------------

macro seek(variable offset) {
  origin (offset & $3FFFFF)
  base offset
}

constant FREE_SPACE($EFFBC8)            // free space used by hack ($11E bytes)
constant lbl_C37083($C37083)            // used for branching
constant lbl_C37047($C37047)            // used for branching

// internal header
seek($C0FFD8)
db $05                                  // SRAM size 5 = 256 Kbits

// rng index
seek($C3016F)
    lda $337F01                         // rng index in SRAM (formely $307FF1)
    inc                                 // rng index +1
    sta $337F01                         // save changes

// game saving
seek($C3151F)
    sta $337F00                         // set last altered file number (formley $307FF0)

// game saving
seek($C31540)
    jsl lbl_sram_save_game              // save game
    jmp lbl_C37083                      // mark all files

// game loading
seek($C3156A)
    jsl lbl_sram_load_game             // load game
    rts

// skins and skin palettes
seek($C31D36)
    lda $337F00                         // last altered file

seek($C33AAA)
    lda $30674E                         // skin slot 1 ($30674E)

seek($C33AC6)
    lda $31674E                         // skin slot 2 ($30714E)

seek($C33AE2)
    lda $32674E                         // skin slot 3 ($307B4E)

seek($C33AF9)
    lda $30674E                         // skin palette slot 1 ($30674E)

seek($C33B0D)
    lda $31674E                         // skin palette slot 2 ($30714E)

seek($C33B21)
    lda $32674E                         // skin palette slot 3 ($307B4E)

// validate SRAM markers
seek($C37028)
    cmp $337F08                         // in file a?
    beq lbl_C37047                      // okay if so
    cmp $337F0A                         // in file b?
    beq lbl_C37047                      // okay if so
    cmp $337F0C                         // in file c?
    beq lbl_C37047                      // okay if so
    cmp $337F0E                         // in file d? (useless but ok...)
    beq lbl_C37047                      // okay if so

// SRAM initialization
seek($C3704B)
    jsl lbl_sram_init_sram              // SRAM initialization
    rts

// file marking as
// Squaresoft property
seek($C37088)
    sta $337F08                         // mark file a (formely $307FF8)
    sta $337F0A                         // mark file b (formely $307FFA)
    sta $337F0C                         // mark file c (formely $307FFC)
    sta $337F0E                         // mark file d (formely $307FFE) (useless but ok...)

//=======================================
// free space code
//=======================================
//=======================================
// game saving
//=======================================
seek(FREE_SPACE)
scope lbl_sram_save_game: {
    cmp #$03                            // is it slot 3?
    beq lbl_save_slot3                  // branch if so
    cmp #$02                            // is it slot 2?
    beq lbl_save_slot2                  // branch if so
//=======================================
// slot 1 saving
//=======================================
    rep #$20
    tdc
    tax
    tay
lbl_save_slot1_sram:
    lda $1600,y                         // regular SRAM
    sta $306000,x                       // copy to slot 1 expanded SRAM
    inx
    inx
    iny
    iny
    cpx #$0A00                          // regular SRAM size
    bne lbl_save_slot1_sram             // loop for all regular SRAM
    tdc
    tax

lbl_save_slot1_sram2:
    lda $336000,x                       // current game SRAM expansion
    sta $306A00,x                       // copy to slot 1 expanded SRAM
    inx
    inx
    cpx #$1600                         // expanded SRAM size
    bne lbl_save_slot1_sram2           // loop for all expanded SRAM
    rtl

//=======================================
// slot 2 saving
//=======================================
lbl_save_slot2:
    rep #$20
    tdc
    tax
    tay
lbl_save_slot2_sram:
    lda $1600,y                         // regular SRAM
    sta $316000,x                       // copy to slot 2 expanded SRAM
    inx
    inx
    iny
    iny
    cpx #$0A00                          // regular SRAM size
    bne lbl_save_slot2_sram             // loop for all regular SRAM
    tdc
    tax

lbl_save_slot2_sram2:
    lda $336000,x                       // current game SRAM expansion
    sta $316A00,x                       // copy to slot 2 expanded SRAM
    inx
    inx
    cpx #$1600                         // expanded SRAM size
    bne lbl_save_slot2_sram2           // loop for all expanded SRAM
    rtl

//=======================================
// slot 3 saving
//=======================================
lbl_save_slot3:
    rep #$20
    tdc
    tax
    tay
lbl_save_slot3_sram:
    lda $1600,y                         // regular SRAM
    sta $326000,x                       // copy to slot 3 expanded SRAM
    inx
    inx
    iny
    iny
    cpx #$0A00                          // regular SRAM size
    bne lbl_save_slot3_sram             // loop for all regular SRAM
    tdc
    tax

lbl_save_slot3_sram2:
    lda $336000,x                       // current game SRAM expansion
    sta $326A00,x                       // copy to slot 3 expanded SRAM
    inx
    inx
    cpx #$1600                         // expanded SRAM size
    bne lbl_save_slot3_sram2           // loop for all expanded SRAM
    rtl
}

//=======================================
// game loading
//=======================================
scope lbl_sram_load_game: {
    cmp #$03                            // is it slot 3?
    beq lbl_load_slot3                  // branch if so
    cmp #$02                            // is it slot 2?
    beq lbl_load_slot2                  // branch if so
//=======================================
// slot 1 loading
//=======================================
    rep #$20
    tdc
    tax
    tay
lbl_load_slot1_sram:
    lda $306000,x                       // load regular slot 1 SRAM
    sta $1600,y                         // save in game SRAM
    inx
    inx
    iny
    iny
    cpx #$0A00                          // regular SRAM size
    bne lbl_load_slot1_sram             // loop for all regular SRAM
    tdc
    tax

lbl_load_slot1_sram2:
    lda $306A00,x                       // load slot 1 expanded SRAM
    sta $336000,x                       // save in game expanded SRAM
    inx
    inx
    cpx #$1600                          // expanded SRAM size
    bne lbl_load_slot1_sram2            // loop for all expanded SRAM
    sep #$20
    rtl

//=======================================
// slot 2 loading
//=======================================
lbl_load_slot2:
    rep #$20
    tdc
    tax
    tay
lbl_load_slot2_sram:
    lda $316000,x                       // load regular slot 2 SRAM
    sta $1600,y                         // save in game SRAM
    inx
    inx
    iny
    iny
    cpx #$0A00                          // regular SRAM size
    bne lbl_load_slot2_sram             // loop for all regular SRAM
    tdc
    tax

lbl_load_slot2_sram2:
    lda $316A00,x                       // load slot 2 expanded SRAM
    sta $336000,x                       // save in game expanded SRAM
    inx
    inx
    cpx #$1600                          // expanded SRAM size
    bne lbl_load_slot2_sram2            // loop for all expanded SRAM
    sep #$20
    rtl

//=======================================
// slot 3 loading
//=======================================
lbl_load_slot3:
    rep #$20
    tdc
    tax
    tay
lbl_load_slot3_sram:
    lda $326000,x                       // load regular slot 3 SRAM
    sta $1600,y                         // save in game SRAM
    inx
    inx
    iny
    iny
    cpx #$0A00                          // regular SRAM size
    bne lbl_load_slot3_sram             // loop for all regular SRAM
    tdc
    tax

lbl_load_slot3_sram2:
    lda $326A00,x                       // load slot 3 expanded SRAM
    sta $336000,x                       // save in game expanded SRAM
    inx
    inx
    cpx #$1600                          // expanded SRAM size
    bne lbl_load_slot3_sram2            // loop for all expanded SRAM
    sep #$20
    rtl
}

//=======================================
// SRAM init
//=======================================
scope lbl_sram_init_sram: {
    rep #$20
    tdc
    tax
init_sram:
    sta $306000,x                       // slot 1
    sta $316000,x                       // slot 2
    sta $326000,x                       // slot 3
    // sta $336000,x                    // you want this line commented likely
                                        // because if you you this RAM for thing
                                        // done at game loading, init is done too late
                                        // and data get erased..
    inx
    inx
    cpx #$2000
    bne init_sram
    sep #$20
    rtl
}
